<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:oxsl="http://www.w3.org/1999/XSL/TransformAlias"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  version="1.0">

  <xsl:output method="xml" omit-xml-declaration="no" version="1.0" indent="yes"/>

  <xsl:namespace-alias stylesheet-prefix="oxsl" result-prefix="xsl"/>

  <xsl:param name="BLOCK-BORDER-GRAY-COLOR">#dddddd</xsl:param>
  <!-- 'Magic' width of the FormBlock table in px; this is used to help the Image 
       Items recalculate their width... A workaround we need until FOP gets smarter. 
       px = (Width of FormBlock in mm) * (dpi / 25.4) 
       ... where dpi = 96.. 
        -->
  <xsl:param name="BLOCK-TABLE-WIDTH">490</xsl:param>
  <xsl:param name="HIBIKI-DEFAULT-FONT-SIZE">8.0</xsl:param>

  <xsl:param name="TITLE-DESC-AREA-PROP">30</xsl:param>

  <!-- TODO: comments for these 2, please. -->
  <xsl:param name="FORM_BASE_DIR"/>

  <xsl:param name="IMG_DIR"/>

  <xsl:template match="/">

    <oxsl:stylesheet version="1.0">

      <oxsl:import href="CUSTOM_XSLT"/>

      <xsl:if test="count(//items/item[@type='RichText']) > 0">
        <!-- The "rich-text" template will be defined in the following imported Stylesheet -->
        <oxsl:import href="RICHTEXT_IMPORT"/>
      </xsl:if>

      <oxsl:output method="xml" omit-xml-declaration="no" version="1.0" indent="yes"/>
      <oxsl:template match="/">
        
        <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format" language="ja">

          <fo:layout-master-set>
            <fo:simple-page-master master-name="mypage"
              page-width="21cm" page-height="297mm"
              margin-left="1.5cm" margin-right="1.5cm"
              margin-top="1.5cm" margin-bottom="1cm">
              <fo:region-body/>
              <fo:region-after/>
            </fo:simple-page-master>
          </fo:layout-master-set>
          
          <fo:page-sequence master-reference="mypage">

            <fo:static-content flow-name="xsl-region-after">
              <fo:block text-align="center">
                <fo:block font-size="{$HIBIKI-DEFAULT-FONT-SIZE}pt">
                  <fo:page-number/> / 
                  <fo:page-number-citation ref-id="last-page"/> 
                </fo:block>
              </fo:block>
            </fo:static-content>

            <fo:flow flow-name="xsl-region-body">
              <oxsl:apply-templates select="//custom-upper-section"/>              
              <fo:block-container>
                <xsl:apply-templates select="form/blocks/block[@type='SimpleLayoutBlock']"/>
              </fo:block-container>
              <oxsl:apply-templates select="//custom-lower-section"/>
              <fo:block id="last-page" line-height="0"></fo:block>
            </fo:flow>

          </fo:page-sequence>

        </fo:root>
        
      </oxsl:template>
      
      <!-- This will generate templates for all items -->
      <xsl:for-each select="//block[@type='SimpleLayoutBlock']">
        <xsl:apply-templates select="items/item"/>
      </xsl:for-each>


      <xsl:apply-templates select="nothing"/>
      <!-- This is to trap unfinished template generators -->
      <oxsl:template match="item">(UNDER CONSTRUCTION)</oxsl:template>

      <!-- Generic template to render accounts. -->
      <oxsl:template match="account">
        <fo:inline>
          <fo:external-graphic content-height="8px">
            <oxsl:attribute name="src">
              <!-- Since the generator of the template knows the image directory, paste it here -->
              <xsl:value-of select="$IMG_DIR"/>
              <oxsl:choose>
                <oxsl:when test="@type = 1 and @status = 8">/ico_fc_userx.gif</oxsl:when>
                <oxsl:when test="@type = 2 and @status = 8">/ico_fc_organization_rpl.gif</oxsl:when>
                <oxsl:when test="@type = 3 and @status = 8">/ico_fc_work_rpl.gif</oxsl:when>
                <oxsl:when test="@type = 4 and @status = 8">/ico_fc_executive_rpl.gif</oxsl:when>
                <oxsl:when test="@type = 1">/ico_fc_user.gif</oxsl:when>
                <oxsl:when test="@type = 2">/ico_fc_organization.gif</oxsl:when>
                <oxsl:when test="@type = 3">/ico_fc_project.gif</oxsl:when>
                <oxsl:when test="@type = 4">/ico_fc_executive.gif</oxsl:when>
              </oxsl:choose>
            </oxsl:attribute>
          </fo:external-graphic>
          <fo:inline color="blue" text-decoration="underline">
            <oxsl:value-of select="name"/>
          </fo:inline>
        </fo:inline>
      </oxsl:template>

      <!-- Generic template to create file icons. -->
      <oxsl:template name="make-file-icon">
        <oxsl:param name="ext" select="default"/>
        <fo:external-graphic content-height="8px">
          <oxsl:attribute name="src">
            <!-- Since the generator of the template knows the image directory, paste it here -->
            <xsl:value-of select="$IMG_DIR"/>
            <oxsl:choose>
              <oxsl:when test="$ext = 'lzh'">/object_zip_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'txt'">/object_txt_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'psd'">/object_psd_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'html' or $ext = 'htm'">/object_html_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'xml'">/object_xml_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'jpg' or $ext = 'jpeg'">/object_img_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'doc'">/object_doc_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'xls'">/object_xls_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'ods'">/object_xls_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'wav'">/object_wav_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'png'">/object_img_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'bmp'">/object_img_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'mpg' or $ext = 'mpeg'">/object_mpg_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'pdf'">/object_pdf_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'ppt'">/object_ppt_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'zip'">/object_zip_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'non'">/object_non_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'ai'">/object_ai_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'gif'">/object_img_xs.gif</oxsl:when>
              <oxsl:when test="$ext = 'cab'">/object_zip_xs.gif</oxsl:when>
            </oxsl:choose>
          </oxsl:attribute>
        </fo:external-graphic>
      </oxsl:template>

    </oxsl:stylesheet>

  </xsl:template>


  <!-- catch-all -->
  <xsl:template match="item"></xsl:template>


  <!-- Specifically match only SimpleLayoutBlocks -->
  <xsl:template match="block[@type='SimpleLayoutBlock']">
    <!-- use the global param BLOCK-BORDER-GRAY-COLOR -->
    <fo:block border-color="{$BLOCK-BORDER-GRAY-COLOR}" border-style="solid" 
      border-width="{number(@frame-style = 1) * 1.0}pt" 
      margin="0pt" space-after="10pt">
      <fo:table>
        <xsl:apply-templates select="columns/column" />
        <fo:table-body>
          <xsl:variable name="colsInBlock" select="columns/@num"/>
          <xsl:apply-templates select="items">
            <xsl:with-param name="cols" select="columns/@num"/>
          </xsl:apply-templates>
        </fo:table-body>
      </fo:table>      
    </fo:block>
  </xsl:template>



  <xsl:template match="items">
    <xsl:param name="cols" select="0"/>
    <xsl:param name="frame-width" select="0"/>
    <!-- Select only the LEADING ITEMs of each row. -->
    <xsl:for-each select="item[(position()-1) mod $cols = 0]">
      <!-- pass each leading ITEM to the row rendering template. -->
      <xsl:call-template name="fullrow-or-singlecell">
        <!-- with the number of columns == items-per-row ("perRow") -->
        <xsl:with-param name="perRow" select="../../columns/@num"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>


  <!-- A recursive template.
       Given the first (leading) ITEM in a row of items, this will render
       the entire row of items.
       This argument to this template is an ITEM which represents the leading
       ITEM in the row. The leading ITEM will be passed with an additional param,
       "perRow", which specifies /how many/ ITEMseach row has.
       This template will first render the leading ITEM, then iterate the
       remaining ITEMs in the row, calling itself recursively. -->
  <xsl:template name="fullrow-or-singlecell">
    <xsl:param name="perRow" select="0"/>
    <!-- Is this ITEM a row leader? -->
    <xsl:choose>
      <xsl:when  test="$perRow > 0">
        <!-- yes, this ITEM is a row leader. -->
        <fo:table-row>
          <!-- first render the row leader. -->
          <xsl:call-template name="fullrow-or-singlecell"/>
          <!-- then select all the remaining ITEMs in this row. -->
          <xsl:for-each select="following-sibling::item[$perRow > position()]">
            <!-- RECURSIVE CALLL: render the remaining items in the row -->
            <xsl:call-template name="fullrow-or-singlecell"/>
          </xsl:for-each>
        </fo:table-row>
      </xsl:when>
      <xsl:otherwise>
        <!-- The current ITEM is NOT a row leader. No further recursion will be needed.  -->
        
        <!-- Generate the ITEM's stylesheet inside the table cell 
             IF the ITEM is non-null -->
        <fo:table-cell xsl:use-attribute-sets="item-table-cell">  <!-- fill in the atrributes from an attr set -->
          <fo:block font-family="MS-Mincho" font-size="{$HIBIKI-DEFAULT-FONT-SIZE}pt">

            <!-- TODO: Wish I did not have to do this xsl:if .. -->
            <xsl:if test="@id">

              <!-- Rendering Strategy:
                   *******************
                   The TITLE or the DESCRIPTION of the Item can be rendered, if specified, in
                   one of the following places marked by the numbers 1, 2, 3, or 4.
                    _________________________        
                   |          1              |   The FORM element in the source tree may contain a TITLE
                   |_________________________|   and/or DESCRIPTION element. These could have a @style
                   |  2  |  ITEM DATA  |  4  |   attribute, which can take values of either 1, 2, 3, or 4.
                   |_____|_____________|_____|   (0 means no display.)
                   |          3              |   This corresponds to the the placement of 1,2,3,4 in the
                   |_________________________|   diagram to the left.
                   
                   The following code will handle this.
                   -->
              <!-- [1] First render, if necessary, the upper portion (labelled as '1') -->
              <xsl:apply-templates select="title[@style=1] | description[@style=1]"/>

              <!-- [2, FormItem, and 4] Then, render the middle portion -->
              <fo:block>
                <!-- The middle block ([2], [FormItem], [4]) is rendered in it's own table. -->
                <fo:table table-layout="fixed" inline-progression-dimension="100%">
                  <!-- The number of columns, and their width, depends on whether the 
                       TITLE and/or DESCRIPTION has been set for either position [2] or [4]
                       -->
                  <!-- COLUMN FOR [2]:  TITLE/DESCRIPTION on the left side? -->
                  <xsl:if test="title[@style=2] or description[@style=2]">
                    <fo:table-column column-width="proportional-column-width({$TITLE-DESC-AREA-PROP})"/>
                  </xsl:if>
                  <!-- COLUMN FOR [FormItem]: This is always specified. -->
                  <fo:table-column column-width="proportional-column-width(100)"/>
                  <!-- COLUMN FOR [4]:  TITLE/DESCRIPTION on the right? -->
                  <xsl:if test="title[@style=4] or description[@style=4]">
                    <fo:table-column column-width="proportional-column-width({$TITLE-DESC-AREA-PROP})"/>
                  </xsl:if>
                  <!-- Finihed setting the columns. -->
                  <fo:table-body>
                    <fo:table-row>
                      <!-- CELL FOR [2]:  Apply the template if applicable either for ITEM, DESCRIPTION -->
                      <xsl:if test="title[@style=2] or description[@style=2]">
                        <fo:table-cell>
                          <xsl:apply-templates select="title[@style=2] | description[@style=2]"/>
                        </fo:table-cell>
                      </xsl:if>
                      <!-- ACTUAL CELL FOR FormItem: Always needed. -->
                      <fo:table-cell>
                        <fo:block>
                          <oxsl:apply-templates select="//item[@id={@id}]"/>
                        </fo:block>
                      </fo:table-cell>
                      <!-- CELL FOR [4]:  Apply the template if applicable either for ITEM, DESCRIPTION -->
                      <xsl:if test="title[@style=4] or description[@style=4]">
                        <fo:table-cell>
                          <xsl:apply-templates select="title[@style=4] |description[@style=4]"/>
                        </fo:table-cell>
                      </xsl:if>
                    </fo:table-row>
                  </fo:table-body>
                </fo:table>
              </fo:block>
              <!-- [2, FormItem, and 4]: MIDDLE portion completed. -->

              <!-- [3] At last, render, if necessary, the lowest portion (labelled as '3') -->
              <xsl:apply-templates select="title[@style=3] | description[@style=3]"/>

            </xsl:if>
          </fo:block>
        </fo:table-cell>
        
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- This will render the description (or the title) text -->
  <xsl:template match="title|description">
    <!-- TODO: hyphenation needs to be configured/added? -->
    <fo:block hyphenate="true" language="en">
      <xsl:attribute name="font-size">
        inherited-property-value(font-size)
        <xsl:choose>
          <xsl:when test="@fontSize = 'fontsizeXlarge'"> +3.0pt </xsl:when>
          <xsl:when test="@fontSize = 'fontsizeLarge'">  +1.5pt </xsl:when>
          <xsl:when test="@fontSize = 'fontsizeMedium'"> +0pt </xsl:when>
          <xsl:when test="@fontSize = 'fontsizeSmall'">  -1.0pt </xsl:when>
          <xsl:when test="@fontSize = 'fontsizeXsmall'"> -2.0pt </xsl:when>
        </xsl:choose>
      </xsl:attribute>
      <xsl:if test="@fontStyle = 'fontStyleItalic'">
        <xsl:attribute name="font-style">italic</xsl:attribute>
      </xsl:if>
      <xsl:if test="@fontWeight = 'fontWeightBlod'">
        <xsl:attribute name="font-weight">bold</xsl:attribute>
      </xsl:if>
      <xsl:if test="@textDecoration = 'textDecorationUnderline'">
        <xsl:attribute name="text-decoration">underline</xsl:attribute>
      </xsl:if>
      <xsl:if test="@color">
        <xsl:attribute name="color"><xsl:value-of select="@color"/></xsl:attribute>
      </xsl:if>
      <xsl:value-of select="text()"/>
    </fo:block>
  </xsl:template>


  <!-- Attribute set for the fo:table-cell that contains 
       ITEM stylesheet templates. The context node will
       be ITEM (//form-items/blocks/block/items/item)
       -->
  <xsl:attribute-set name="item-table-cell">
    <xsl:attribute name="border-width">1.0pt</xsl:attribute>
    <xsl:attribute name="border-color">
      <xsl:choose> <!-- Set the border color to gray IF (@frame-style == 1) -->
        <xsl:when test="../../@frame-style = 1">
          <xsl:value-of select="$BLOCK-BORDER-GRAY-COLOR"/>
        </xsl:when>        
        <xsl:otherwise>white</xsl:otherwise> <!-- No frame style; white border -->
      </xsl:choose>
    </xsl:attribute>
    <xsl:attribute name="border-style">solid</xsl:attribute>
    <xsl:attribute name="background-color">
      <xsl:value-of select="@bgcolor"/> <!-- Set the bg color -->
    </xsl:attribute>
  </xsl:attribute-set>

  <xsl:template match="columns/column">
    <xsl:choose>
      <xsl:when test="@width > 0">
        <xsl:variable name="colWidth" select="@width"/> 
        <fo:table-column column-width="{$colWidth}%"/>
      </xsl:when>
      <xsl:otherwise>
        <fo:table-column/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- Templates for rendering the individual stylesheet components of each FormItem. -->

  <xsl:template 
    match="item[@type='Text' or @type='SelectLongText' or @type='SelectInput' or @type='Counter']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <oxsl:value-of select="data/value"/>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='DynaChangeableLabel']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <xsl:choose>
        <xsl:when test="data-type = 1"> <!-- Number -->
          <fo:block text-align="right">
            <!-- As of now, no units: Just a formatted, right-aligned number -->
            <oxsl:value-of select="format-number(data/value, '{number-format}')"/>
          </fo:block>
        </xsl:when>
        <xsl:otherwise> <!-- Others: Could be Text, Date (Date not implemented yet)-->
          <oxsl:value-of select="data/value"/>     
        </xsl:otherwise>
      </xsl:choose>
    </oxsl:template>
  </xsl:template>


  <xsl:template  match="item[@type='LongText']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block linefeed-treatment="preserve">
        <oxsl:value-of select="data/value"/>   
      </fo:block>
    </oxsl:template>
  </xsl:template>


  <xsl:template  match="item[@type='RichText']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block>
        <oxsl:apply-templates select="data/rich-text"/>
      </fo:block>
    </oxsl:template>
  </xsl:template>


  <!-- Rendering Image items is a little tricky because FOP has scaling issues. -->
  <xsl:template match="item[@type='Image']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block>
        <!-- TODO: Image scaling needs work. FOP 0.92 is still not working as expected. -->
        <fo:external-graphic 
          src="/var/hibiki/data/{{data/absoluteFilename}}">
          <oxsl:attribute name="content-width">
            <oxsl:variable name="max-permissible-width">
              <!-- Calculate the explicit value to set to this image (using "content-width") -->
              <!-- This happens in 2 parts: 
                   (1) Calculate the width using the width of the OUTER TABLE (SimpleLayoutBlock) column 
                   (2) Calculate any further scaling IF the ITEM TITLE/DESC also need some width
                   Do (1) first: -->
              <!-- Find out WHICH column number this ITEM falls under. -->
              <xsl:variable name="col">
                <xsl:variable name="_col" select="position() mod count(../../columns/column)"/>
                <xsl:choose>
                  <xsl:when test="$_col = 0">
                    <xsl:value-of select="count(../../columns/column)"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$_col"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <!-- Using the known width of the outer table (BLOCK-TABLE-WIDTH) in px, calculate the width in px -->
              <xsl:variable name="widthmm">
                <xsl:variable name="_widthper" select="../../columns/column[position()=$col]/@width"/>
                <xsl:choose>
                  <xsl:when test="$_widthper = -1">
                    <xsl:value-of select="($BLOCK-TABLE-WIDTH div count(../../columns/column))"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="($BLOCK-TABLE-WIDTH div 100) * $_widthper"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <!-- Now, do (2): -->
              <!-- Re-Calculate the WIDTH attribute for the fop image. This is needed because 
                   we have to resize according to the inner table as well. 
                   Use this formula: 
                   if _EITHER_  AREA2 _OR_  AREA4  then   widthmm * (100 / (100 + 30*1) )
                   if _BOTH_    AREA2 _AND_ AREA4  then   widthmm * (100 / (100 + 30*2) ) 
                   if _NEITHER_ AREA2 _NOR_ AREA4  then   widthmm * (100 / 100) /* no change */ -->
              <xsl:variable name="area2-prop" select="number(title[@style='2'] or description[@style='2'])"/>
              <xsl:variable name="area4-prop" select="number(title[@style='4'] or description[@style='4'])"/>
              <xsl:variable name="more-than-hundred" select="($area2-prop + $area4-prop) * $TITLE-DESC-AREA-PROP"/>
              <!-- Finally, the last calculation; return the width in mm  -->
              <xsl:value-of select="$widthmm * (100 div ($more-than-hundred + 100))"/>
            </oxsl:variable>
            <oxsl:choose>
              <oxsl:when test="data/@width &gt; $max-permissible-width">
                <oxsl:value-of select="$max-permissible-width"/>
              </oxsl:when>
              <oxsl:otherwise>
                <oxsl:value-of select="data/@width"/>
              </oxsl:otherwise>
            </oxsl:choose>
            <oxsl:text>px</oxsl:text>
          </oxsl:attribute>
        </fo:external-graphic>
        <!-- TODO: should we setup a constant for '/var/hibiki/data'?  -->
      </fo:block>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='Link']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:inline color="blue" text-decoration="underline">
        <oxsl:value-of select="data/title"/>
      </fo:inline>
      <fo:inline font-style="italic"> 
        (<oxsl:value-of select="data/url"/>)
      </fo:inline>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='CheckBox' or @type='Radio' or @type='Select']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <!--  just a simple render-all.. -->
      <oxsl:for-each select="data/selectedMap">
        <oxsl:if test="not(position() = 1)">, </oxsl:if>
        <oxsl:value-of select="value"/>
      </oxsl:for-each>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='Date']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <oxsl:for-each select="data"> <!-- Of course, there is only 1 "data" element -->
        <xsl:choose>
          <xsl:when test="mode = 4">
            <!-- HH:mm -->
            <oxsl:value-of select="format-number(hour,'00')"/>:<oxsl:value-of select="format-number(minutes,'00')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:if test="mode >= 1">
              <!-- yyyy/MM -->
              <oxsl:value-of select="year"/>/<oxsl:value-of select="format-number(month,'00')"/>
            </xsl:if>
            <xsl:if test="mode >= 2">
              <!-- yyyy/MM/dd -->
               <oxsl:text>/</oxsl:text><oxsl:value-of select="format-number(day,'00')"/>
            </xsl:if>
            <xsl:if test="mode >= 3">
              <!-- yyyy/MM/dd HH:mm -->
              <fo:character character="&#x20;"/>
              <oxsl:value-of select="format-number(hour,'00')"/>:<oxsl:value-of select="format-number(minutes,'00')"/>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
      </oxsl:for-each>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='Number']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block text-align="right">
        <xsl:value-of select="unitStringLeft"/>
        <oxsl:value-of select="format-number(data/value, '{number-format}')"/>
        <xsl:value-of select="unitStringRight"/>
      </fo:block>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='Label']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <oxsl:value-of select="value"/>
      <!-- seems like nothing else is needed here -->
    </oxsl:template>
  </xsl:template>


  <!-- We have 2 kinds of DesignImages; defined by the "type" element: 
       type:file, or type url. -->

  <!-- DesignImage that uses a file stored on the filesystem -->
  <xsl:template match="item[@type='DesignImage' and type='file']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <!-- This is so much easier than the Struts/Velocity version! -->
      <fo:external-graphic content-height="{height}" content-width="{width}">
        <xsl:attribute name="src">
          <!-- Image SRC is built by concatenating the SmartForm's base dir and "i" + formId + ".dat" -->
          <xsl:value-of select="$FORM_BASE_DIR"/>/i<xsl:value-of select="@id"/>.dat
        </xsl:attribute>
      </fo:external-graphic>
    </oxsl:template>
  </xsl:template>


  <!-- DesignImage that uses a url to display the image -->
  <xsl:template match="item[@type='DesignImage' and type='url']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <!-- This is so much easier than the Struts/Velocity version! -->
      <fo:external-graphic src="{url}" content-height="50%"/>
    </oxsl:template>
  </xsl:template>
  

  <xsl:template match="item[@type='DesignLink' and link-style='buttonlink']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block padding="3pt">
        <fo:inline>&#160;</fo:inline>
        <!-- Make this look like a button -->
        <fo:inline
          background-color="#cccccc"
          border-left-color="#dddddd"   border-left-width="2pt"   border-left-style="solid"
          border-top-color="#dddddd"    border-top-width="2pt"    border-top-style="solid"
          border-right-color="#bbbbbb"  border-right-width="2pt"  border-right-style="solid"
          border-bottom-color="#bbbbbb" border-bottom-width="2pt" border-bottom-style="solid">
          <xsl:value-of select="link-title"/>
        </fo:inline>
        <xsl:if test="link-type = 'url'">
          <fo:inline font-style="italic">(<xsl:value-of select="link-url"/>)</fo:inline>
        </xsl:if>
      </fo:block>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='DesignLink' and link-style='textlink']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:inline color="blue" text-decoration="underline">
        <xsl:value-of select="link-title"/>
      </fo:inline>
      <xsl:if test="link-type = 'url'">
        <fo:inline font-style="italic">(<xsl:value-of select="link-url"/>)</fo:inline>
      </xsl:if>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='AccountList' or @type='AccountChoice']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <oxsl:for-each select="data/account-list/account">
        <oxsl:if test="not(position() = 1)">, </oxsl:if>
        <oxsl:apply-templates select="."/>
      </oxsl:for-each>
    </oxsl:template>
  </xsl:template>
  

  <xsl:template match="item[@type='FileFolder']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <oxsl:for-each select="data/file-list/file">
        <fo:block>
          <oxsl:call-template name="make-file-icon">
            <oxsl:with-param name="ext" select="extension"/>
          </oxsl:call-template>
          <fo:inline>&#160;</fo:inline>
          <oxsl:value-of select="name"/>   
        </fo:block>
      </oxsl:for-each>
    </oxsl:template>
  </xsl:template>


  <xsl:template match="item[@type='DataRecordSelector']">
    <oxsl:template match="item[@id={@id}]">
      <oxsl:variable name="id" select="{@id}"/>
      <fo:block padding="2pt">
        <fo:inline>&#160;</fo:inline>
        <fo:external-graphic src="{$IMG_DIR}/ico_hibiki_docsearch.gif" content-height="8px"/>
        <fo:inline>&#160;</fo:inline>
        <fo:inline color="blue" text-decoration="underline">
          <oxsl:value-of select="data/title"/>
        </fo:inline>      
      </fo:block>
    </oxsl:template>
  </xsl:template>
  
  

</xsl:stylesheet>
