// 一括操作メニュー
var BulkOperationMenu = Class.create({
	initialize : function(container) {
		this.container = $(container);
	},

	refresh : function(data) {
		// cleanup old contents
		this.container.update('');
		this.tabGroupIds = new Array();
		if(data.length == 0) { // when data is empty, we need show alert message
			this.createEmptyMenu();
		} else {
			this.createMenu(data);
		}
		return false;
	},

	createMenu : function(data) {
		this._data = data;
		var menuPane;
		if(window.isUseUnifiedUI) {
			menuPane = new Element('div', {id : "contents_main", style :"padding:7px", 'class': "unifiedPanel"});
		} else {
			menuPane = new Element('div', {id : "contents_main", style : "border:1px solid #523E32;background-color:#FFFFFF;padding:7px"});
		}
		
		this.container.insert(menuPane);
		// menu groups
		for (var i=0; i<data.length; i++) {
			this.createGroup(menuPane, data[i]);
		}

	},
	
	createEmptyMenu : function() {
		var description = "$text.get('bulkOperation.emptyMenu.message','BRD_Operations')<br /><br />";
		if(window.isUseUnifiedUI) {
			menuPane = new Element('div', {id : "contents_main", style :"padding:7px", 'class': "unifiedPanel"});
		} else {
			menuPane = new Element('div', {id : "contents_main", style : "border:1px solid #523E32;background-color:#FFFFFF;padding:7px"});
		}
		menuGroup = new Element('div', {id : 'empty'});
		menuPane.insert(menuGroup);
		var header = new Element('div', {'class' : "kadomaru_box box_color02"});
		header.insert(new Element('div', {'class' : "box_top"}).insert(new Element('div')));
		if(window.isUseUnifiedUI){
			header.insert(getUnifiedIconElement("unifiedSearchIcon", 36, {style: "float: left;"}));
		}else{
			header.insert(new Element('img', {src : hibikiImageUrl("smartdb/operations/icon_search"), align : "left"}));
		}
		header.insert(new Element('p', {'class' : "box_caption caption_color02"}));
		header.insert(new Element('p', {'class' : "description"}).update(description));
		header.insert(new Element('div', {'class' : "box_bottom"}).insert(new Element('div')));
		menuGroup.insert(header);
		this.container.insert(menuPane);
	},

	/**
	 * data in group array specification:
     *  [{
     *      id: group id,
     *      headCss: group header css class, default is "kadomaru_box box_color02",
     *      img: group image, default is "images/smartdb/operations/icon_search.gif",
     *      captionCss: group caption css class, default is "box_caption caption_color02",
     *      caption: group caption,
     *      description: group description,
     *      tabs: [{
     *      	id: tab id,
     *      	label: tab label
     *      },...],
     *      subGroups: [{
     *      	id: subGroup id,
     *      	label: subGroup label
     *      },...],
     *      buttons: [{
     *      	css: button css class, default is "linkmenu  hover01",
     *      	label: button label,
     *      	img: button image, optional,
     *      	description: button description, optional,
     *      	url: menu button url,
     *      	execButtons: [{
     *          	label: execute button label,
     *          	url: execute button url
     *      	}, ...]
     *      },...]
     *  },...],
     *  
     * @param params 
     */
	createGroup : function(parent, group) {
		menuGroup = new Element('div', {id : group.id});
		parent.insert(menuGroup);
		var header = new Element('div', {'class' : group.headCss ? group.headCss : "kadomaru_box box_color02"});
		header.insert(new Element('div', {'class' : "box_top"}).insert(new Element('div')));
		if(window.isUseUnifiedUI){
			header.insert(getUnifiedIconElement("unifiedSearchIcon", 36, {style: "float: left;"}));
		}else{
			header.insert(new Element('img', {src : hibikiImageUrl(group.img) || hibikiImageUrl("smartdb/operations/icon_search"), align : "left"}));
		}
		header.insert(new Element('p', {'class' : group.captionCss || "box_caption caption_color02"}).update(group.caption));
		header.insert(new Element('p', {'class' : "description"}).update(group.description));
		header.insert(new Element('div', {'class' : "box_bottom"}).insert(new Element('div')));
		menuGroup.insert(header);


		// Tabs
		if (group.tabs) {
			this.createTabs(group.tabs);
		}

		// SubGroups
		if (group.subGroups) {
			for (var i=0; i<group.subGroups.length; i++) {
				this.createSubGroup(group.subGroups[i]);
			}
		}

		// Buttons
		if (group.buttons) {
			for (var i=0; i<group.buttons.length; i++) {
				this.createButton(group.buttons[i]);
			}
		}
	},

	createTabs : function(tabs) {
		var parentId = tabs[0].parentId;
		var tabGroupId = parentId + "_tabs";
		var tabGroup = new Element('ul', {id : tabGroupId, 'class' : "tabs"});
		this.tabGroupIds.push(tabGroupId);
		$(parentId).insert(new Element('div', {'class' : "tabmenu"}).insert(tabGroup));
		$(parentId).insert(new Element('br',{style : "clear:both;"}));
		for (var i=0; i<tabs.length; i++) {
			var tab = tabs[i];
			var elm = new Element('li').insert(new Element('a', {href : "#" + tab.id}).insert(new Element('span').update(tab.label)));
			console.log("tab.id:"+tab.id);
			tabGroup.insert(elm);
			$(parentId).insert(new Element('div', {id : tab.id}));
			// SubGroups
			if (tab.subGroups) {
				for (var i=0; i<tab.subGroups.length; i++) {
					this.createSubGroup(tab.subGroups[i]);
				}
			}
			// Buttons
			if (tab.buttons) {
				for (var i=0; i<tab.buttons.length; i++) {
					this.createButton(tab.buttons[i]);
				}
			}
		}
		// init Control.Tabs
		try {
			new Control.Tabs(tabGroupId);
		} catch(e) {
			alert("Exception:\n"+e);
		}
	},

	createSubGroup : function(sub) {
		// サブグループは開く、閉じることができる
		var subGroupHeadId = sub.id + "_head";
		var subGroupHead = new Element('div', {'class' : "process_caption", id : subGroupHeadId}).update(sub.label.escapeHTML());
		subGroupHead.insert({top: new Element('img', {src : hibikiImageUrl("smartdb/operations/icon_minus")})});
		if (!$(sub.parentId)) {
			alert("No such element id: " + sub.parentId);
			return;
		}
		$(sub.parentId).insert(subGroupHead);
		$(sub.parentId).insert(new Element('div', {id : sub.id}));

		// Buttons
		if (sub.buttons) {
			for (var i=0; i<sub.buttons.length; i++) {
				this.createButton(sub.buttons[i]);
			}
		}
		subGroupHead.observe('click', function() {
			if ($(sub.id).visible()) {
				$(sub.id).hide();
				$(subGroupHead).down('img').src = hibikiImageUrl('smartdb/operations/icon_plus');
			} else {
				$(sub.id).show();
				$(subGroupHead).down('img').src = hibikiImageUrl('smartdb/operations/icon_minus');
			}
		});
	},
	
	_getButtonId: function (button) {
		return button.id;
	},
	
	/**
	 * var p = {
	 * 	group: ...,
	 *  subgroup: ...
	 *  button: ....
	 * }
	 * 
	 * o.getButton().hide()
	 */
	getButton: function(p) {
		if (typeof p === "string") {
			// means: p is button id
		} else if (typeof p === "object") {
			// get group; get tab, get subgroup, then button...
		}
		var div = null;
		// TODO: buttonのdivを取得する
		div = $(p);
		return div; // div.show(), div.hide() は可能
	},
	
	
	createButton : function(button) {
		var menuItem = new Element('div', {
			id: this._getButtonId(button),
			'class' : button.css || "linkmenu hover01"
		});
		$(button.parentId).insert(menuItem);
		var itemStyle;
		var item;
		if (button.img && menuItem.id.indexOf('doc') == -1) {
			itemStyle = "padding-left:30px; padding-right:1em;background-image:url(" + hibikiImageUrl(button.img)+");background-size:16px;";
		} else if(button.img && menuItem.id.indexOf('doc') != -1){
			itemStyle = "padding-left:30px; padding-right:1em;padding-top:6px;padding-bottom:6px;background-image:url(" + hibikiImageUrl(button.img)+");background-size:15px;background-repeat:no-repeat;background-position:12px center;";
		} else {
			itemStyle = "padding-left:30px; padding-right:1em;";
		}
		if(menuItem.id.indexOf('doc') != -1){
			item = new Element('div', {style : itemStyle}).update(button.label);
		} else{
			item = new Element('a', {style : itemStyle, href : "#"}).update(button.label);
		}		
		menuItem.insert(new Element('ul').insert(new Element('li').insert(item)));
		if (button.description) {
			item.insert({bottom : new Element('span', {'class' : "text_color01"}).update('&nbsp;：&nbsp;' + button.description)});
		}
		item.onclick = function() {
			if (typeof selectBulkOperation == 'function')
				return selectBulkOperation(button);
			else
				return false;
		};
	}

});
