// $Id: DataEditor.js 73637 2015-01-07 09:20:33Z q_fang $
// super class for various data bulk editors

var ElementEditor = Class.create({
	initialize : function(parent, options, selector, index) {
		var tr = $(parent.insertRow(parent.rows.length));
		tr.id = 'Item_' + options.id;
		tr.hide();
		var td = $(tr.insertCell(0));
		td.className = 'parts_name';
		var span = new Element('span').update(options.title.escapeHTML());
		td.insert(span);
		if (options.mustInputFlag) {
			td.insert(new Element('span').setStyle({
				color : 'red'
			}).update("*"));
		}
		td = $(tr.insertCell(1));
		td.className = 'parts_detail';
		td = $(tr.insertCell(2));
		td.className = 'parts_del';
		var p = new Element('p');
		p.className = 'delbtn';
		p.style.position="static";
		td.insert(p);
		var a = new Element('a');
		p.insert(a);
		a.href = "#";
		a.onclick = this.remove.bind(this);
		var img = new Element('img', {
			src : hibikiImageUrl('smartdb/operations/remove_button'),
			border : 0,
			width : 23,
			height : 38
		});
		a.insert(img);
		td = $(tr.insertCell(3));
		td.hide();
		var anchor = new Element('a');
		anchor.onclick = this.toJSON.bind(this);
		td.insert(anchor);
		this.tr = tr;
		this.selector = selector;
		this.options = options
		this.inAnimation = false;
		this.index = index;
		this.data = options.data;
		// this.options = options;
	},

	remove : function() {
		if (!this.inAnimation) {
			this.tr.fade({
				duration : 0.1,
				beforeStartInternal : function(effect) {
					this.inAnimation = true;
				}.bind(this),
				afterFinishInternal : function(effect) {
					this.tr.hide();
					this.inAnimation = false;
				}.bind(this)
			});
			
			this.clear();
			
			var beforeOp = $A(this.selector.options).find(function(elm) {
				return this.index < elm.getAttribute('orgIndex');
			}.bind(this));
			var option = new Element('option', {
				value : this.options.id,
				'orgIndex' : this.index
			}).update(this.options.title.escapeHTML());
			if (beforeOp == null)
				this.selector.insert(option);
			else {
				this.selector.insertBefore(option, beforeOp);
			}
		}
		return false;
	},

	toJSON : function() {
		return "{itemId: " + this.options.id + ", " + this.getSetValueString()
				+ "}";
	},
	getWtrimedValue : function() {
		return this.simpleEditArea.value = wtrim(this.simpleEditArea.value);
	},
	getSetValueString : function() {
		return 'value: ' + this.simpleEditArea.value.toJSON();
	},
	setMessage: function() {
		if(this.options.message) {
			td = $(this.tr.cells[1]);
			td.insert("<div class='ErrorMessage'>" + this.options.message + "</div>");
		}
	},
	clear: function() {
		// do nothing
	}

});

var CategoryEditor = Class
		.create(
				ElementEditor,
				{
					initialize : function($super, parent, options, selector,
							index) {
						$super(parent, options, selector, index);
						td = $(this.tr.cells[1]);
						var innerHtml = "";
						innerHtml += "<input type='hidden' name='filter_"
								+ options.id;
						innerHtml += "' id='filter_" + options.id;
						innerHtml += "' /><div id='categorySelectorContainer_"
								+ options.id;
						innerHtml += "'>      </div>";
						td.innerHTML = innerHtml;
						var target;
						
						var startCategoryId;
						if(this.data && this.data.selectedCategoryId) {
							startCategoryId = this.data.selectedCategoryId;
						} else {
							startCategoryId = options.classificationId;
						}
						
						target = new DA.SmartDB.Widget.CategorySelector(
								$('categorySelectorContainer_' + options.id),
								startCategoryId,
								options.classificationId, null, options.id,
								true);
						target.onSet = function(newCategoryId) {
							$('filter_' + options.id).value = newCategoryId;
						}
						target.setErrorMessage('');
						this.simpleEditArea = target;
						this.option_id = options.id;
						this.topId = options.classificationId;
						$('categorySelectorContainer_' + options.id).down('td').style.border = "0px";
						$('categorySelectorContainer_' + options.id).down('td').style.paddingTop = "0px";
						$('categorySelectorContainer_' + options.id).down('td').style.paddingBottom = "0px";
						$('categorySelectorContainer_' + options.id).down('td',
								1).style.border = "0px";
						$('categorySelectorContainer_' + options.id).down('td',
								1).style.paddingTop = "0px";
						$('categorySelectorContainer_' + options.id).down('td',
								1).style.paddingBottom = "0px";
					},
					getSetValueString : function() {
						return "value: '" + $('filter_' + this.option_id).value
								+ "'";
					},
					getWtrimedValue : function() {
						return $('filter_' + this.option_id).value = wtrim($('filter_'
								+ this.option_id).value);
					},
					clear : function() {
						var tthis = this.simpleEditArea;
						tthis.isBuildablePath(tthis.topObjId, tthis.topObjId, {
							callback: function (flg) {
								tthis.isBuildablePathFlg=flg;
								tthis.callBrowseBundle(tthis.topObjId, tthis.topObjId, {
									callback: function (bundle) {
										tthis.renderBrowse(bundle);
										tthis.updateButton('edit');
										tthis.pane.hide();
										if (bundle.parentTree) {
											if (tthis.canSelect(bundle.parentTree[tthis.typeName])) {
												tthis.errorMsg.hide();
												tthis._onSet(bundle.parentTree[tthis.typeName]);
											} else {
												tthis.errorMsg.show();
												tthis._onUnSet();
											}
										} else {
											tthis._onSet(null);
										}
									},
									errorHandler: tthis.genericErrorHandler
								});
							},
							errorHandler: tthis.genericErrorHandler
						});
					}
				});

var TextEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);
		// simple editor
		var text = new Element('input', {
			type : 'text',
			name : 'value' + options.id,
			style : 'width: ' + options.size * 8 + 'px'
		});
		if(this.data) {
			text.value = this.data.value;
		}
		td.insert(text);
		this.simpleEditArea = text;
	},
	clear : function() {
		this.simpleEditArea.value = "";
	}
});

var NumberEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);
		var text = new Element('input', {
			type : 'text',
			name : 'value' + options.id,
			style : 'width: ' + options.size * 8 + 'px'
		});
		if(this.data) {
			text.value = this.data.value;
		}
		td.insert(text);
		this.simpleEditArea = text;
	},
	clear : function() {
		this.simpleEditArea.value = "";
	}
});

var AccountChoiceEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);
		// simple editor
		var select = new Element('select', {
			name : 'value' + options.id
		});
		this.simpleEditArea = select;
		td.insert(select);
		
		var selectedAcctId;
		if(this.data) {
			var acctList = this.data.accountDataList;
			if(acctList.length > 0) {
				selectedAcctId = acctList[0].id;
			}
		}
		
		for (var i = 0; i < options.selectableAccounts.length; i++) {
			var acct = options.selectableAccounts[i];
			var opt = new Element('option', {
				value : acct.id
			}).update(acct.name.escapeHTML());
			if(selectedAcctId == acct.id) {
				opt.selected = "selected";
			}
			select.insert(opt);
		}

	},
	getSetValueString : function() {
		return "midList: '" + this.simpleEditArea.value + "'";
	},
	clear : function() {
		this.simpleEditArea.selectedIndex = 0;
	}
});

var AccountListEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);
		// simple editor
		var accountSelectorId = 'accountList' + options.id;
		var div = new Element('div').addClassName('AccountListPicker');
		this.simpleEditArea = div;
		td.insert(div);
		var span = new Element('span', {
			id : accountSelectorId
		});
		div.insert(span);
		span = new Element('span', {
			style : "float: right;"
		}).addClassName('btn_icon');
		div.insert(span);
		var a = new Element('a', {
			href : "#"
		});
		span.insert(a);
		this.selectButton = a;
		span = new Element('span').update(checkLang.selectButton);
		a.insert(span);
		a.style.position="static";
		span.style.position="static";

		var accountSelectorInstance = new Widget.AccountSelector(
				accountSelectorId);
		a.onclick = this.openAccountSelectDialog.bind(this,
				accountSelectorInstance, parseInt(options.selectType),
				parseInt(options.maxSelected));
		
		if(this.data) {
			var acctList = this.data.accountDataList;
			for(var i = 0; i < acctList.length; i++) {
				var acct = acctList[i];
				accountSelectorInstance.add(acct.id, acct.name.escapeHTML(), acct.type, acct.status, false);
			}
		}
	},

	openAccountSelectDialog : function(accountSelectorInstance, type,
			maxSelected) {
		accountSelectorInstance.openSelectorPopup(type, maxSelected, 0);
		return false;
	},

	getInputValue : function() {
		var value = new Array();
		this.simpleEditArea.select('down', 'input').each(function(elm) {
			return value.push(elm.value);
		})
		return value.toString();
	},

	getSetValueString : function() {
		var value = this.getInputValue();
		return "midList: '" + value + "'";
	},
	
	clear : function() {
		this.simpleEditArea.down("span").innerHTML = "";;
	}
});

var RadioEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);
		// this.simpleEditArea = div;
		var table = new Element('table').addClassName('choiceInput');
		td.insert(table);

		var span;
		var tr = table.insertRow(0);
		var cellNum = 0;
		
		var selectedMap;
		if(this.data) {
			selectedMap = this.data.selectedMap;
		}
		
		for (var i = 0; i < options.sortedOptionList.length; i++) {
			var choice = options.sortedOptionList[i];
			if (!choice.disabled) {
				if (cellNum == options.columns) {
					tr = table.insertRow(table.rows.length);
					cellNum = 0
				}
				td = $(tr.insertCell(cellNum))
				span = new Element('span', {
					style : 'white-space: nowrap;'
				});
				radio = new Element('input', {
					type : 'radio',
					value : choice.id,
					name : 'value' + options.id
				});

				span.appendChild(radio);
				
				if(selectedMap && selectedMap[choice.id]) {
					//radio.checked = "checked";
					radio.setAttribute("checked","checked");
				}
				
				span.appendChild(document.createTextNode(choice.name));
				td.insert(span);
				cellNum++;
			}
		}
		this.simpleEditArea = table;
	},

	getInputValue : function() {
		var value = new Array();
		this.simpleEditArea.select('down', 'input').each(function(elm) {
			if (elm.checked)
				return value.push(elm.value);
		})
		return value.toString();
	},

	getSetValueString : function() {
		var value = this.getInputValue();
		return "optionList:'" + value + "'";
	},
	
	clear : function() {
		this.simpleEditArea.select('down', 'input').each(function(elm) {
			elm.checked = false;
		})
	}
});

var CheckBoxEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// value cell
		td = $(this.tr.cells[1]);

		var table = new Element('table').addClassName('choiceInput');
		td.insert(table);

		var span;
		var tr = table.insertRow(0);
		var cellNum = 0;
		
		var selectedMap;
		if(this.data) {
			selectedMap = this.data.selectedMap;
		}
		
		for (var i = 0; i < options.sortedOptionList.length; i++) {
			var choice = options.sortedOptionList[i];
			if (!choice.disabled) {
				if (cellNum == options.columns) {
					tr = table.insertRow(table.rows.length);
					cellNum = 0
				}
				td = $(tr.insertCell(cellNum))
				span = new Element('span', {
					style : 'white-space: nowrap;'
				});
				checkbox = new Element('input', {
					type : 'checkbox',
					value : choice.id,
					name : 'value' + options.id
				});

				span.appendChild(checkbox);
				
				if(selectedMap && selectedMap[choice.id]) {
					//checkbox.checked = "checked";
					checkbox.setAttribute("checked","checked");
				}
				
				span.appendChild(document.createTextNode(choice.name));
				td.insert(span);
				cellNum++;
			}
		}
		this.simpleEditArea = table;
	},

	getInputValue : function() {
		var value = new Array();
		this.simpleEditArea.select('down', 'input').each(function(elm) {
			if (elm.checked)
				return value.push(elm.value);
		})
		return value.toString();
	},

	getSetValueString : function() {
		var value = this.getInputValue();
		return "optionList: '" + value.toString() + "'";
	},
	
	clear : function() {
		this.simpleEditArea.select('down', 'input').each(function(elm) {
			elm.checked = false;
		})
	}
});

var SelectEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// operation mode cell

		td = $(this.tr.cells[1]);
		// simple editor
		var select = new Element('select', {
			name : 'value' + options.id
		});
		this.simpleEditArea = select;
		td.insert(select);
		
		var selectedMap;
		if(this.data) {
			selectedMap = this.data.selectedMap;
		}
		
		for (var i = 0; i < options.sortedOptionList.length; i++) {
			var choice = options.sortedOptionList[i];
			if (!choice.disabled) {
				var opt = new Element('option', {
					value : choice.id
				}).update(choice.name.escapeHTML());
				
				if(selectedMap && selectedMap[choice.id]) {
					opt.selected = "selected";
				}
				
				select.insert(opt);
			}
		}
	},
	getSetValueString : function() {
		var value = this.simpleEditArea.value;
		return "optionList: '" + value.toString() + "'";
	},
	
	clear : function() {
		this.simpleEditArea.selectedIndex = 0;
	}
});

var SelectInputEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		// operation mode cell
		// value cell
		td = $(this.tr.cells[1]);
		// simple editor

		var table = new Element('table', {
			cellspacing : 0,
			cellpadding : 0,
			border : 0
		}).addClassName('choiceInput');
		td.insert(table);
		var tr = table.insertRow(0)
		td = $(tr.insertCell(0));
		var text = new Element('input', {
			type : 'text',
			name : 'value' + options.id,
			style : 'vertical-align: top; width:' + options.size * 8 + 'px'
		});
		
		if(this.data) {
			text.value = this.data.value;
		}
		
		td.insert(text);
		td = $(tr.insertCell(1));
		this.simpleEditArea = text;
		var anchor = new Element('a', {
			href : '#'
		});
		td.insert(anchor);
		anchor.onclick = this.openSelector.bind(this);
		var img = new Element('img', {
			height : 22,
			width : 22,
			src : hibikiImageUrl('aqbtn_pcm_down_b')
		});
		anchor.insert(img);
		this.offsetA = anchor;
		var selector = new Element('div', {
			id : 'selector' + options.id,
			style : 'display: none'
		}).addClassName('selectInput1');
		this.selectorDiv = selector;
		td.insert(selector);
		table = new Element('table', {
			cellspacing : 0,
			cellpadding : 0,
			border : 0
		});
		selector.insert(table);
		var i;
		var rowNum = 0;
		for (i = 0; i < options.sortedOptionList.length; i++) {
			var choice = options.sortedOptionList[i];
			if (!choice.disabled) {
				anchor = new Element('a', {
					href : '#'
				}).update(choice.name.escapeHTML());

				anchor.onclick = this.selectData.bind(this, choice.name);
				$(table.insertRow(rowNum).insertCell(0)).insert(anchor);
				rowNum++;
			}
		}
		var div = new Element('div');
		div.className = 'SelectInputLine';
		img = new Element('img', {
			height : 1,
			width : 1,
			src : 'images/null.gif'
		})
		div.insert(img);
		$(table.insertRow(rowNum).insertCell(0)).insert(div);

	},
	selectData : function(value) {
		this.simpleEditArea.value = value;
		this.selectorDiv.hide();
		closeShim(this.selectorDiv);
		return false;
	},
	openSelector : function() {
		if (this.selectorDiv.style.display == 'none') {
			var td = this.selectorDiv.up('td');
			var inputTable = td.up('td');
			var table = this.tr.up('table');
			this.selectorDiv.style.left = td.offsetLeft + inputTable.offsetLeft
					+ table.offsetLeft + 'px';
			this.selectorDiv.style.top = td.offsetTop + inputTable.offsetTop
					+ table.offsetTop + 32 + 'px';
			this.selectorDiv.show();
			//openShim(this.selectorDiv);
		} else {
			this.selectorDiv.hide();
			//closeShim(this.selectorDiv);
		}
		// 全画面のDivControllerのレジスターに登録する
		singlePopupDivController.addFloatDiv(this.selectorDiv);
		return false;
	},
	
	clear : function() {
		this.simpleEditArea.value = "";
	}
});

var DateEditor = Class.create(ElementEditor, {
	initialize : function($super, parent, options, selector, index) {
		$super(parent, options, selector, index);
		td = $(this.tr.cells[1]);
		var select;
		var option;
		var idKey = 'Date_' + options.id;
		var divEl = new Element('div');
		var me = this;
		if (options.displayMode != 4) {
			// year
			var yearSelEl, i = 0, innerHtml = new Array(
					options.yearRange.length + 1);

			yearSelEl = "<select id='Date_" + options.id + "_Y'>";
			innerHtml[i] = "<option value='-1'>----</option>";
			options.yearRange.each(function(year) {
				i++;
				var selected = "";
				if(me.data && me.data.year == year) {
					selected = "selected='selected'";
				}
				innerHtml[i] = "<option value='" + year + "' " + selected + ">" + year + "</option>";
			});
			yearSelEl += innerHtml.join() + '</select>';
			divEl.innerHTML = yearSelEl;
			divEl.appendChild(document.createTextNode(checkLang.yearLable));

			// month
			select = new Element('select', {
				id : 'Date_' + options.id + '_Mon'
			});
			option = new Element('option', {
				value : -1
			}).update("--");
			select.insert(option);
			for (var i = 1; i < 13; i++) {
				option = new Element('option', {
					value : i
				}).update(i < 10 ? '0' + i : i);
				
				if(me.data && me.data.month == i) {
					option.selected = "selected";
				}
				
				select.insert(option);
			}

			divEl.insert(select).appendChild(
					document.createTextNode(checkLang.monthLable));
		}
		if (options.displayMode == 2 || options.displayMode == 3) {
			// month
			select = new Element('select', {
				id : 'Date_' + options.id + '_D'
			});
			option = new Element('option', {
				value : -1
			}).update("--");
			select.insert(option);
			for (var i = 1; i < 32; i++) {
				option = new Element('option', {
					value : i
				}).update(i < 10 ? '0' + i : i);
				
				if(me.data && me.data.day == i) {
					option.selected = "selected";
				}
				
				select.insert(option);
			}

			divEl.insert(select).appendChild(
					document.createTextNode(checkLang.dayLable));

			var anchor = new Element('a', {
				id : idKey + '_Picker'
			});
			anchor.href = 'javascript:void(0);';

			divEl.insert(anchor);

			var img = new Element('img', {
				src : hibikiImageUrl("calendar/ico_hibiki_calendar"),
				border : '0',
				align : 'absmiddle'
			});
			anchor.insert(img);
			var div = new Element('div', {
				id : idKey + '_Cal',
				style : 'display:none'
			});
			div.className = 'calInDiv';

			divEl.insert(div);

			calList[idKey] = new Cal(div.id, "calList['" + idKey + "']", idKey,
					options.lang, "calOnSetBulkEdit(calList['" + idKey
							+ "'], '" + idKey + "')");
			anchor.onclick = this.calOpen.bind(anchor, calList[idKey], div.id,
					anchor.id);
		}
		// cal
		if (options.displayMode == 3 || options.displayMode == 4) {
			// hour
			select = new Element('select', {
				id : 'Date_' + options.id + '_h'
			});
			option = new Element('option', {
				value : -1
			}).update("--");
			select.insert(option);
			for (var i = 0; i < 24; i++) {
				option = new Element('option', {
					value : i
				}).update(i < 10 ? '0' + i : i);
				
				if(me.data && me.data.hour == i) {
					option.selected = "selected";
				}
				
				select.insert(option);
			}

			divEl.insert(select).appendChild(
					document.createTextNode(checkLang.hourLable));

			select = new Element('select', {
				id : 'Date_' + options.id + '_m'
			});
			option = new Element('option', {
				value : -1
			}).update("--");
			select.insert(option);
			for (var i = 0; i < 60; i++) {
				option = new Element('option', {
					value : i
				}).update(i < 10 ? '0' + i : i);
				
				if(me.data && me.data.minute == i) {
					option.selected = "selected";
				}
				
				select.insert(option);
			}

			divEl.insert(select).appendChild(
					document.createTextNode(checkLang.minuteLable));
			// td.insert(select);
			// td.insert(checkLang.minuteLable);// todo
		}
		td.insert(divEl);
	},
	getSetValueString : function() {
		var value = '';
		var hasValue = false;
		this.tr.select('down', 'select').each(function(elm) {
			hasValue = hasValue || (elm.value != -1);
		});
		if (hasValue) {
			var select = 'Date_' + this.options.id + '_Y';
			if ($(select)) {
				value += ($(select).value == -1 ? '0000' : $(select).value);
			}
			select = 'Date_' + this.options.id + '_Mon';
			if ($(select)) {
				value += '/' + ($(select).value == -1 ? 0 : $(select).value);
			}
			select = 'Date_' + this.options.id + '_D';
			if ($(select)) {
				value += '/' + ($(select).value == -1 ? 0 : $(select).value);
			}
			select = 'Date_' + this.options.id + '_h';
			if ($(select)) {
				if (value.length != 0)
					value += ' '
				value += ($(select).value == -1 ? 0 : $(select).value);
			}
			select = 'Date_' + this.options.id + '_m';
			if ($(select))
				value += ':' + ($(select).value == -1 ? 0 : $(select).value);
		}
		return "value: '" + value + "'";
	},
	calOpen : function(_calobj, _calid, _pickerid, _ev) {
		var cal = $(_calid);
		var picker = $(_pickerid);
		fncPopup2(_ev, _calobj, _calid, _pickerid);
		if (cal.style.display == 'block') {
			cal.style.left = picker.offsetLeft + picker.up('td').offsetLeft
					+ 20 + 'px';
			cal.style.top = picker.offsetTop + picker.up('td').offsetTop
					+ picker.up('table').offsetTop + 17 + 'px';
			openShim(cal);
		}
		return false;
	},
	
	clear : function() {
		this.tr.select('down', 'select').each(function(elm) {
			elm.selectedIndex = 0;
		});
	}
});

// FormItem selector
var ItemSelector = Class.create({

	initialize : function(list, selector, dialog) {
		this.list = $(list);
		this.list.style.listStyleType = 'none';
		this.list.style.listStylePosition = 'inside';
		this.selector = $(selector);
		this.selector.onchange = this.add.bind(this);
		this.selector.onmousewheel = doNothing;
		this.index = 0;
		this.dialog = $(dialog);
	},

	addItem : function(itemId, options) {
		this.index++;
		var option = new Element('option', {
			value : options.id,
			'orgIndex' : this.index
		}).update(options.title.escapeHTML());
		this.selector.insert(option);
		try {
			var editor;
			switch (options.typeDescriptor) {
			case 'Text':
				editor = new TextEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'Number':
				editor = new NumberEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'LongText':
				editor = new TextAreaEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'AccountChoice':
				editor = new AccountChoiceEditor(this.list, options,
						this.selector, this.index);
				break;
			case 'AccountList':
				editor = new AccountListEditor(this.list, options,
						this.selector, this.index);
				break;
			case 'Radio':
				editor = new RadioEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'CheckBox':
				editor = new CheckBoxEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'Select':
				editor = new SelectEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'SelectInput':
				editor = new SelectInputEditor(this.list, options,
						this.selector, this.index);
				break;
			case 'Date':
				editor = new DateEditor(this.list, options, this.selector,
						this.index);
				break;
			case 'Category':
				editor = new CategoryEditor(this.list, options, this.selector,
						this.index);
				break;
			}
			if(editor) {
				editor.setMessage();
			}
		} catch (e) {
			alert(options.typeDescriptor + ":  " + e);
		}
		return false;
	},

	add : function() {
		if (this.selector.value != "-1") {
			var id = 'Item_' + this.selector.value;
			var orgWidth = this.dialog.offsetWidth;
			$(id).appear({
				duration : 0.1,
				afterFinishInternal : function(effect) {
					if (orgWidth < this.list.offsetWidth) {
						this.dialog.style.width = this.list.offsetWidth + 40;
					}
				}.bind(this)
			});
			this.selector.remove([ this.selector.selectedIndex ]);
			this.selector.selectedIndex = 0;
		}
	},
	
	showItem: function(id, index) {
		id = 'Item_' + id;
		var orgWidth = this.dialog.offsetWidth;
		$(id).appear({
			duration : 0.1,
			afterFinishInternal : function(effect) {
				if (orgWidth < this.list.offsetWidth) {
					this.dialog.style.width = this.list.offsetWidth + 40;
				}
			}.bind(this)
		});
		this.selector.remove([ index ]);
	}

});